@@ -10,10 +10,14 @@
 #include "qsocketnotifier.h"
 
 #include <QtCore/qlist.h>
-#include <QtCore/qpair.h>
+
+#include <QtCore/q26numeric.h>
 
 #include <glib.h>
 
+using namespace std::chrono;
+using namespace std::chrono_literals;
+
 QT_BEGIN_NAMESPACE
 
 struct GPollFDWithQSocketNotifier
@@ -95,11 +99,13 @@ struct GTimerSource
 
 static gboolean timerSourcePrepareHelper(GTimerSource *src, gint *timeout)
 {
-    timespec tv = { 0l, 0l };
-    if (!(src->processEventsFlags & QEventLoop::X11ExcludeTimers) && src->timerList.timerWait(tv))
-        *timeout = (tv.tv_sec * 1000) + ((tv.tv_nsec + 999999) / 1000 / 1000);
-    else
+    if (src->processEventsFlags & QEventLoop::X11ExcludeTimers) {
         *timeout = -1;
+        return true;
+    }
+
+    auto remaining = src->timerList.timerWait().value_or(-1ms);
+    *timeout = q26::saturate_cast<gint>(ceil<milliseconds>(remaining).count());
 
     return (*timeout == 0);
 }
@@ -110,10 +116,7 @@ static gboolean timerSourceCheckHelper(GTimerSource *src)
         || (src->processEventsFlags & QEventLoop::X11ExcludeTimers))
         return false;
 
-    if (src->timerList.updateCurrentTime() < src->timerList.constFirst()->timeout)
-        return false;
-
-    return true;
+    return !src->timerList.hasPendingTimers();
 }
 
 static gboolean timerSourcePrepare(GSource *source, gint *timeout)
@@ -319,18 +322,21 @@ QEventDispatcherGlibPrivate::QEventDispatcherGlibPrivate(GMainContext *context)
     g_source_attach(&idleTimerSource->source, mainContext);
 }
 
+QEventDispatcherGlibPrivate::~QEventDispatcherGlibPrivate()
+    = default;
+
 void QEventDispatcherGlibPrivate::runTimersOnceWithNormalPriority()
 {
     timerSource->runWithIdlePriority = false;
 }
 
 QEventDispatcherGlib::QEventDispatcherGlib(QObject *parent)
-    : QAbstractEventDispatcher(*(new QEventDispatcherGlibPrivate), parent)
+    : QAbstractEventDispatcherV2(*(new QEventDispatcherGlibPrivate), parent)
 {
 }
 
 QEventDispatcherGlib::QEventDispatcherGlib(GMainContext *mainContext, QObject *parent)
-    : QAbstractEventDispatcher(*(new QEventDispatcherGlibPrivate(mainContext)), parent)
+    : QAbstractEventDispatcherV2(*(new QEventDispatcherGlibPrivate(mainContext)), parent)
 { }
 
 QEventDispatcherGlib::~QEventDispatcherGlib()
@@ -338,7 +344,7 @@ QEventDispatcherGlib::~QEventDispatcherGlib()
     Q_D(QEventDispatcherGlib);
 
     // destroy all timer sources
-    qDeleteAll(d->timerSource->timerList);
+    d->timerSource->timerList.clearTimers();
     d->timerSource->timerList.~QTimerInfoList();
     g_source_destroy(&d->timerSource->source);
     g_source_unref(&d->timerSource->source);
@@ -475,10 +481,11 @@ void QEventDispatcherGlib::unregisterSocketNotifier(QSocketNotifier *notifier)
     }
 }
 
-void QEventDispatcherGlib::registerTimer(int timerId, qint64 interval, Qt::TimerType timerType, QObject *object)
+void QEventDispatcherGlib::registerTimer(Qt::TimerId timerId, Duration interval,
+                                         Qt::TimerType timerType, QObject *object)
 {
 #ifndef QT_NO_DEBUG
-    if (timerId < 1 || interval < 0 || !object) {
+    if (qToUnderlying(timerId) < 1 || interval < 0ns || !object) {
         qWarning("QEventDispatcherGlib::registerTimer: invalid arguments");
         return;
     } else if (object->thread() != thread() || thread() != QThread::currentThread()) {
@@ -488,14 +495,13 @@ void QEventDispatcherGlib::registerTimer(int timerId, qint64 interval, Qt::Timer
 #endif
 
     Q_D(QEventDispatcherGlib);
-    d->timerSource->timerList.registerTimer(timerId, std::chrono::milliseconds{ interval },
-                                            timerType, object);
+    d->timerSource->timerList.registerTimer(timerId, interval, timerType, object);
 }
 
-bool QEventDispatcherGlib::unregisterTimer(int timerId)
+bool QEventDispatcherGlib::unregisterTimer(Qt::TimerId timerId)
 {
 #ifndef QT_NO_DEBUG
-    if (timerId < 1) {
+    if (qToUnderlying(timerId) < 1) {
         qWarning("QEventDispatcherGlib::unregisterTimer: invalid argument");
         return false;
     } else if (thread() != QThread::currentThread()) {
@@ -524,28 +530,30 @@ bool QEventDispatcherGlib::unregisterTimers(QObject *object)
     return d->timerSource->timerList.unregisterTimers(object);
 }
 
-QList<QEventDispatcherGlib::TimerInfo> QEventDispatcherGlib::registeredTimers(QObject *object) const
+QList<QEventDispatcherGlib::TimerInfoV2> QEventDispatcherGlib::timersForObject(QObject *object) const
 {
+#ifndef QT_NO_DEBUG
     if (!object) {
-        qWarning("QEventDispatcherUNIX:registeredTimers: invalid argument");
-        return QList<TimerInfo>();
+        qWarning("QEventDispatcherGlib:timersForObject: invalid argument");
+        return {};
     }
+#endif
 
     Q_D(const QEventDispatcherGlib);
     return d->timerSource->timerList.registeredTimers(object);
 }
 
-int QEventDispatcherGlib::remainingTime(int timerId)
+QEventDispatcherGlib::Duration QEventDispatcherGlib::remainingTime(Qt::TimerId timerId) const
 {
 #ifndef QT_NO_DEBUG
-    if (timerId < 1) {
+    if (qToUnderlying(timerId) < 1) {
         qWarning("QEventDispatcherGlib::remainingTimeTime: invalid argument");
-        return -1;
+        return Duration::min();
     }
 #endif
 
-    Q_D(QEventDispatcherGlib);
-    return d->timerSource->timerList.timerRemainingTime(timerId);
+    Q_D(const QEventDispatcherGlib);
+    return d->timerSource->timerList.remainingDuration(timerId);
 }
 
 void QEventDispatcherGlib::interrupt()
@@ -570,7 +578,7 @@ bool QEventDispatcherGlib::versionSupported()
 }
 
 QEventDispatcherGlib::QEventDispatcherGlib(QEventDispatcherGlibPrivate &dd, QObject *parent)
-    : QAbstractEventDispatcher(dd, parent)
+    : QAbstractEventDispatcherV2(dd, parent)
 {
 }
 
