@@ -27,9 +27,9 @@ void QPropertyBindingPrivatePtr::reset(QtPrivate::RefCounted *ptr) noexcept
 {
     if (ptr != d) {
         if (ptr)
-            ptr->ref++;
+            ptr->addRef();
         auto *old = std::exchange(d, ptr);
-        if (old && (--old->ref == 0))
+        if (old && !old->deref())
             QPropertyBindingPrivate::destroyAndFreeMemory(static_cast<QPropertyBindingPrivate *>(d));
     }
 }
@@ -161,7 +161,7 @@ struct QPropertyDelayedNotifications
         delayed->d_ptr = 0;
 
         if (observer)
-            observer.notify<QPropertyObserverPointer::Notify::OnlyChangeHandlers>(delayed->propertyData);
+            observer.notify(delayed->propertyData);
     }
 };
 
@@ -185,7 +185,7 @@ Q_CONSTINIT static thread_local QBindingStatus bindingStatus;
     properties need to be updated, preventing any external observer from noticing an inconsistent
     state.
 
-    \sa Qt::endPropertyUpdateGroup
+    \sa Qt::endPropertyUpdateGroup, QScopedPropertyUpdateGroup
 */
 void Qt::beginPropertyUpdateGroup()
 {
@@ -205,7 +205,7 @@ void Qt::beginPropertyUpdateGroup()
     \warning Calling endPropertyUpdateGroup without a preceding call to beginPropertyUpdateGroup
     results in undefined behavior.
 
-    \sa Qt::beginPropertyUpdateGroup
+    \sa Qt::beginPropertyUpdateGroup, QScopedPropertyUpdateGroup
 */
 void Qt::endPropertyUpdateGroup()
 {
@@ -226,8 +226,8 @@ void Qt::endPropertyUpdateGroup()
         data = data->next;
     }
     // notify all delayed notifications from binding evaluation
-    for (const QBindingObserverPtr &observer: bindingObservers) {
-        QPropertyBindingPrivate *binding = observer.binding();
+    for (const auto &bindingPtr: bindingObservers) {
+        auto *binding = static_cast<QPropertyBindingPrivate *>(bindingPtr.get());
         binding->notifyNonRecursive();
     }
     // do the same for properties which only have observers
@@ -235,12 +235,45 @@ void Qt::endPropertyUpdateGroup()
     while (data) {
         for (qsizetype i = 0; i < data->used; ++i)
             data->notify(i);
-        auto *next = data->next;
-        delete data;
-        data = next;
+        delete std::exchange(data, data->next);
     }
 }
 
+/*!
+    \since 6.6
+    \class QScopedPropertyUpdateGroup
+    \inmodule QtCore
+    \ingroup tools
+    \brief RAII class around Qt::beginPropertyUpdateGroup()/Qt::endPropertyUpdateGroup().
+
+    This class calls Qt::beginPropertyUpdateGroup() in its constructor and
+    Qt::endPropertyUpdateGroup() in its destructor, making sure the latter
+    function is reliably called even in the presence of early returns or thrown
+    exceptions.
+
+    \note Qt::endPropertyUpdateGroup() may re-throw exceptions thrown by
+    binding evaluations. This means your application may crash
+    (\c{std::terminate()} called) if another exception is causing
+    QScopedPropertyUpdateGroup's destructor to be called during stack
+    unwinding. If you expect exceptions from binding evaluations, use manual
+    Qt::endPropertyUpdateGroup() calls and \c{try}/\c{catch} blocks.
+
+    \sa QProperty
+*/
+
+/*!
+    \fn QScopedPropertyUpdateGroup::QScopedPropertyUpdateGroup()
+
+    Calls Qt::beginPropertyUpdateGroup().
+*/
+
+/*!
+    \fn QScopedPropertyUpdateGroup::~QScopedPropertyUpdateGroup()
+
+    Calls Qt::endPropertyUpdateGroup().
+*/
+
+
 // check everything stored in QPropertyBindingPrivate's union is trivially destructible
 // (though the compiler would also complain if that weren't the case)
 static_assert(std::is_trivially_destructible_v<QPropertyBindingSourceLocation>);
@@ -277,7 +310,7 @@ void QPropertyBindingPrivate::unlinkAndDeref()
 {
     clearDependencyObservers();
     propertyDataPtr = nullptr;
-    if (--ref == 0)
+    if (!deref())
         destroyAndFreeMemory(this);
 }
 
@@ -288,27 +321,12 @@ bool QPropertyBindingPrivate::evaluateRecursive(PendingBindingObserverList &bind
     return evaluateRecursive_inline(bindingObservers, status);
 }
 
-void QPropertyBindingPrivate::notifyRecursive()
-{
-    if (!pendingNotify)
-        return;
-    pendingNotify = false;
-    Q_ASSERT(!updating);
-    updating = true;
-    if (firstObserver) {
-        firstObserver.noSelfDependencies(this);
-        firstObserver.notify(propertyDataPtr);
-    }
-    if (hasStaticObserver)
-        staticObserverCallback(propertyDataPtr);
-    updating = false;
-}
-
 void QPropertyBindingPrivate::notifyNonRecursive(const PendingBindingObserverList &bindingObservers)
 {
     notifyNonRecursive();
-    for (auto &&bindingObserver: bindingObservers) {
-        bindingObserver.binding()->notifyNonRecursive();
+    for (auto &&bindingPtr: bindingObservers) {
+        auto *binding = static_cast<QPropertyBindingPrivate *>(bindingPtr.get());
+        binding->notifyNonRecursive();
     }
 }
 
@@ -321,7 +339,7 @@ QPropertyBindingPrivate::NotificationState QPropertyBindingPrivate::notifyNonRec
     updating = true;
     if (firstObserver) {
         firstObserver.noSelfDependencies(this);
-        firstObserver.notifyOnlyChangeHandler(propertyDataPtr);
+        firstObserver.notify(propertyDataPtr);
     }
     if (hasStaticObserver)
         staticObserverCallback(propertyDataPtr);
@@ -621,9 +639,11 @@ void QPropertyBindingData::notifyObservers(QUntypedPropertyData *propertyDataPtr
             if (storage)
                 d = QPropertyBindingDataPointer {storage->bindingData(propertyDataPtr)};
             if (QPropertyObserverPointer observer = d.firstObserver())
-                observer.notifyOnlyChangeHandler(propertyDataPtr);
-            for (auto &&bindingObserver: bindingObservers)
-                bindingObserver.binding()->notifyNonRecursive();
+                observer.notify(propertyDataPtr);
+            for (auto &&bindingPtr: bindingObservers) {
+                auto *binding = static_cast<QPropertyBindingPrivate *>(bindingPtr.get());
+                binding->notifyNonRecursive();
+            }
         }
     }
 }
@@ -658,11 +678,15 @@ QPropertyObserver::QPropertyObserver(ChangeHandler changeHandler)
     d.setChangeHandler(changeHandler);
 }
 
+#if QT_DEPRECATED_SINCE(6, 6)
 QPropertyObserver::QPropertyObserver(QUntypedPropertyData *data)
 {
+    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
     aliasData = data;
     next.setTag(ObserverIsAlias);
+    QT_WARNING_POP
 }
+#endif
 
 /*! \internal
 */
@@ -732,16 +756,9 @@ void QPropertyObserverPointer::setChangeHandler(QPropertyObserver::ChangeHandler
     ptr->next.setTag(QPropertyObserver::ObserverNotifiesChangeHandler);
 }
 
-void QPropertyObserverPointer::setBindingToNotify(QPropertyBindingPrivate *binding)
-{
-    Q_ASSERT(ptr->next.tag() != QPropertyObserver::ObserverIsPlaceholder);
-    ptr->binding = binding;
-    ptr->next.setTag(QPropertyObserver::ObserverNotifiesBinding);
-}
-
 /*!
     \internal
-    The same as as setBindingToNotify, but assumes that the tag is already correct.
+    The same as setBindingToNotify, but assumes that the tag is already correct.
  */
 void QPropertyObserverPointer::setBindingToNotify_unsafe(QPropertyBindingPrivate *binding)
 {
@@ -791,9 +808,11 @@ void QPropertyObserverPointer::evaluateBindings(PendingBindingObserverList &bind
         if (QPropertyObserver::ObserverTag(observer->next.tag()) == QPropertyObserver::ObserverNotifiesBinding) {
             auto bindingToEvaluate = observer->binding;
             QPropertyObserverNodeProtector protector(observer);
-            QBindingObserverPtr bindingObserver(observer); // binding must not be gone after evaluateRecursive_inline
-            if (bindingToEvaluate->evaluateRecursive_inline(bindingObservers, status))
-                bindingObservers.push_back(std::move(bindingObserver));
+            // binding must not be gone after evaluateRecursive_inline
+            QPropertyBindingPrivatePtr currentBinding(observer->binding);
+            const bool evalStatus = bindingToEvaluate->evaluateRecursive_inline(bindingObservers, status);
+            if (evalStatus)
+                bindingObservers.push_back(std::move(currentBinding));
             next = protector.next();
         }
 
@@ -1266,15 +1285,13 @@ QString QPropertyBindingError::description() const
   can be used to express relationships between different properties in your
   application.
 
-  \note In the case of QML it is important that \l QProperty needs to be exposed
-  in \l Q_PROPERTY with the BINDABLE keyword. As a result the QML engine, uses it
-  as the bindable interface to set up the property binding. In turn, the binding
-  can be then interacted with C++ via the normal API like:
-
+  \note For QML, it's important to expose the \l QProperty in \l Q_PROPERTY
+  with the BINDABLE keyword. As a result, the QML engine uses
+  it as the bindable interface to set up the property binding. In turn, the
+  binding can then be interacted with C++ via the normal API:
   QProperty<T>::onValueChanged, QProperty::takeBinding and QBindable::hasBinding
-
-  If the property is BINDABLE, then the engine will use the change-tracking
-  inherent to the C++ property system for getting notified about changes; and
+  If the property is BINDABLE, the engine will use the change-tracking
+  inherent to the C++ property system for getting notified about changes, and it
   won't rely on signals being emitted.
 */
 
@@ -1349,15 +1366,6 @@ QString QPropertyBindingError::description() const
   Assigns \a newValue to this property and returns a reference to this QProperty.
 */
 
-/*!
-  \fn template <typename T> QProperty<T> &QProperty<T>::operator=(const QPropertyBinding<T> &newBinding)
-
-  Associates the value of this property with the provided \a newBinding
-  expression and returns a reference to this property. The property's value is
-  set to the result of evaluating the new binding. Whenever a dependency of the
-  binding changes, the binding will be re-evaluated.
-*/
-
 /*!
   \fn template <typename T> QPropertyBinding<T> QProperty<T>::setBinding(const QPropertyBinding<T> &newBinding)
 
@@ -2022,26 +2030,6 @@ QString QPropertyBindingError::description() const
   QPropertyAlias.
 */
 
-/*!
-  \fn template <typename T> QPropertyAlias<T> &QPropertyAlias<T>::operator=(T &&newValue)
-  \overload
-
-  Assigns \a newValue to the aliased property and returns a reference to this
-  QPropertyAlias.
-*/
-
-/*!
-  \fn template <typename T> QPropertyAlias<T> &QPropertyAlias<T>::operator=(const QPropertyBinding<T> &newBinding)
-  \overload
-
-  Associates the value of the aliased property with the provided \a newBinding
-  expression and returns a reference to this alias. The property's value is set
-  to the result of evaluating the new binding. Whenever a dependency of the
-  binding changes, the binding will be re-evaluated, and the property's value
-  gets updated accordingly.
-
-*/
-
 /*!
   \fn template <typename T> QPropertyBinding<T> QPropertyAlias<T>::setBinding(const QPropertyBinding<T> &newBinding)
 
@@ -2494,8 +2482,13 @@ QPropertyAdaptorSlotObject::QPropertyAdaptorSlotObject(QObject *o, const QMetaPr
 {
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(7, 0, 0)
 void QPropertyAdaptorSlotObject::impl(int which, QSlotObjectBase *this_, QObject *r, void **a,
                                       bool *ret)
+#else
+void QPropertyAdaptorSlotObject::impl(QSlotObjectBase *this_, QObject *r, void **a, int which,
+                                      bool *ret)
+#endif
 {
     auto self = static_cast<QPropertyAdaptorSlotObject *>(this_);
     switch (which) {
