@@ -49,7 +49,7 @@ void tst_QPlugin::initTestCase()
 
 void tst_QPlugin::loadDebugPlugin()
 {
-    const auto fileNames = dir.entryList(QStringList() << "*debug*", QDir::Files);
+    const auto fileNames = dir.entryList(QStringList() << "*debugplugin*", QDir::Files);
     if (fileNames.isEmpty())
         QSKIP("No debug plugins found - skipping test");
 
@@ -82,7 +82,7 @@ void tst_QPlugin::loadDebugPlugin()
 
 void tst_QPlugin::loadReleasePlugin()
 {
-    const auto fileNames = dir.entryList(QStringList() << "*release*", QDir::Files);
+    const auto fileNames = dir.entryList(QStringList() << "*releaseplugin*", QDir::Files);
     if (fileNames.isEmpty())
         QSKIP("No release plugins found - skipping test");
 
@@ -180,7 +180,7 @@ void tst_QPlugin::scanInvalidPlugin()
 #if defined(Q_OS_MACOS) && defined(Q_PROCESSOR_ARM)
     QSKIP("This test crashes on ARM macOS");
 #endif
-    const auto fileNames = dir.entryList({"*invalid*"}, QDir::Files);
+    const auto fileNames = dir.entryList({"*invalidplugin*"}, QDir::Files);
     QString invalidPluginName;
     if (fileNames.isEmpty())
         QSKIP("No invalid plugin found - skipping test");
