@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -84,7 +84,7 @@ void tst_QCalendarWidget::getSetCheck()
     QDate selectedDate(2005, 7, 3);
     QSignalSpy spy(&object, SIGNAL(selectionChanged()));
     object.setSelectedDate(selectedDate);
-    QCOMPARE(spy.count(), 1);
+    QCOMPARE(spy.size(), 1);
     QCOMPARE(selectedDate, object.selectedDate());
     //month and year
     object.setCurrentPage(2004, 1);
@@ -110,11 +110,19 @@ void tst_QCalendarWidget::getSetCheck()
     object.setSelectedDate(selectedDate);
     QCOMPARE(minDate, object.selectedDate());
     QVERIFY(selectedDate != object.selectedDate());
+    object.clearMinimumDate();
+    object.setSelectedDate(selectedDate);
+    QCOMPARE(selectedDate, object.selectedDate());
+
     //date should not go beyond the maximum.
     selectedDate = maxDate.addDays(10);
     object.setSelectedDate(selectedDate);
     QCOMPARE(maxDate, object.selectedDate());
     QVERIFY(selectedDate != object.selectedDate());
+    object.clearMaximumDate();
+    object.setSelectedDate(selectedDate);
+    QCOMPARE(selectedDate, object.selectedDate());
+
     //show today
     QDate today = QDate::currentDate();
     object.showToday();
